// generated by script
#include "bind_consts.h"


void bind_consts(py::module &m) {
    py::module consts = m.def_submodule("consts", "API Consts");

    {% for const in data_consts -%}
    consts.attr("{{ const['name'] }}") = {{ const['value'].replace("'", '"') }};
    {% endfor %}
}